/** \file main.c
	\author Tomasz Ostrowski
	\date August 2007
	
	Main file of SHT11 sensor library test project.
	Compiler: avr-gcc (WinAVR 20060421)
	Based on Sensirion sample code app note, rev. 2.05
	MCU: ATmega8, 4MHz crystal, Vcc = 5V (important for SHT11 calculations)
	sensor: SHT11, DATA with external 10k pullup connected to PD4, SCK connected to PD3
	MCU <-> PC: UART
	HFUSE/LFUSE: 0xC9/0x9F
*/

#include "config.h"	       //IMPORTANT: it contains F_CPU definition, so include it before delay.h
#include <avr/wdt.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>


#include "lcd/HD44780.h"

#include "sht/libsht.h"

#include "rfm12/rfm12.h"

static unsigned int sekunden = 0;

long temperatur, feuchtigkeit, taupunkt;

// Starten des Timers, nachdem er in der Hauptschleife konfiguriert wurde
uint8_t timer_start(void)
{
 TIMSK1 = (1<<OCIE1A);   //Interrupt bei Compare Match
 sei();            //Interrputs aktivieren
 return 0;
}
  
ISR(TIMER1_COMPA_vect)
{
 sekunden++;
}

//Bei einem unerwarteten Interrupt nichts machen:
void __attribute__ ((naked)) __vector_default(void) {__asm__ ("reti");}

// Hilfsvariable LCD nur einmal aktualisieren
unsigned char anzeigen = (-1);

int main(void)
{
 datenblock block;
 int temp1,temp2,temp3,temp4;
 
 uchar err;

 rfm12_init();
 _delay_ms(500);							// Initialisierung muss manchmal mehrmals gemacht werden,
 rfm12_init();								// da Modul lange braucht
 _delay_ms(500);							// Initialisierung muss manchmal mehrmals gemacht werden,
 rfm12_init();

//rfm12_setfreq(RF12FREQ(433.92));	        // Sende/Empfangsfrequenz auf 433,92MHz einstellen
 rfm12_setfreq(RF12FREQ(868.90));	
 rfm12_setbandwidth(4, 1, 4);			    // 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -79dBm 
 rfm12_setbaud(4800);						// 19200 baud
 rfm12_setpower(0, 6);						// 1mW Ausgangangsleistung, 120kHz Frequenzshift
 
	
	lcd_init(LCD_DISP_ON);
    lcd_clrscr();
	
  char buf[20];

		
//16Bit-Timer fr Zeitzhlung
  TCCR1B |= (1 << WGM12);                //CTC Modus
  TCCR1B |= (1 << CS12) | (0 << CS11) | (0 << CS10);   //Prescaler 256
  OCR1A = 31249;                      //Compare Wert setzen
  timer_start();
  
//Bei einem unerwarteten Interrupt nichts machen:
void __attribute__ ((naked)) __vector_default(void) {__asm__ ("reti");}
  
  
	sei(); //interrupts freigeben

    while (1) 
    {
	
void anzeige (void)
{
		lcd_clrscr();
		
       if(temperatur%100 < 10)
       {		
		lcd_gotoxy(0,0);
        sprintf( buf, "%ld.0%ld C", temperatur/100, temperatur%100 );
		lcd_puts(buf);
	   }
       else
       {
		lcd_gotoxy(0,0);
        sprintf( buf, "%ld.%ld C", temperatur/100, temperatur%100 );
		lcd_puts(buf);
	   }
	
       if(feuchtigkeit%100 < 10)
       {		
		lcd_gotoxy(0,1);
        sprintf( buf, "%ld.0%ld RH", feuchtigkeit/100, feuchtigkeit%100 );
		lcd_puts(buf);
	   }
       else
       {
		lcd_gotoxy(0,1);
        sprintf( buf, "%ld.%ld RH", feuchtigkeit/100, feuchtigkeit%100 );
		lcd_puts(buf);
	   }
	
       if(taupunkt%100 < 10)
       {		
		lcd_gotoxy(0,2);
        sprintf( buf, "%ld.0%ld C", taupunkt/100, taupunkt%100 );
		lcd_puts(buf);
	   }
       else
       {
		lcd_gotoxy(0,2);
        sprintf( buf, "%ld.%ld C", taupunkt/100, taupunkt%100 );
		lcd_puts(buf);
	   }
	   
       if(temp1%100 < 10)
       {		
		lcd_gotoxy(12,0);
        sprintf( buf, "%i.0%i C", temp1/100, temp1%100 );
		lcd_puts(buf);
	   }
       else
       {
		lcd_gotoxy(12,0);
        sprintf( buf, "%i.%i C", temp1/100, temp1%100 );
		lcd_puts(buf);
	   }
	   
       if(temp2%100 < 10)
       {		
		lcd_gotoxy(12,1);
        sprintf( buf, "%i.0%i RH", temp2/100, temp2%100 );
		lcd_puts(buf);
	   }
       else
       {
		lcd_gotoxy(12,1);
        sprintf( buf, "%i.%i RH", temp2/100, temp2%100 );
		lcd_puts(buf);
	   }

       if(temp3%100 < 10)
       {		
		lcd_gotoxy(12,2);
        sprintf( buf, "%i.0%i C", temp3/100, temp3%100 );
		lcd_puts(buf);
	   }
       else
       {
		lcd_gotoxy(12,2);
        sprintf( buf, "%i.%i C", temp3/100, temp3%100 );
		lcd_puts(buf);
	   }
	   
       if(temp4%100 < 10)
       {		
		lcd_gotoxy(13,3);
        sprintf( buf, "%i.0%i V", temp4/100, temp4%100 );
		lcd_puts(buf);
	   }
       else
       {
		lcd_gotoxy(13,3);
        sprintf( buf, "%i.%i V", temp4/100, temp4%100 );
		lcd_puts(buf);
	   }
	   

		lcd_gotoxy(0,3);
		sprintf( buf, "%i", err );
		lcd_puts(buf); 

		lcd_gotoxy(5,3);
		sprintf( buf, "%i", sekunden );
		lcd_puts(buf);		
}
	
   
        if(sekunden == 60) sekunden = 0;
		
	
    err=rfm12_empfang(block.data);
	temp1=block.temp1;
	temp2=block.temp2;
	temp3=block.temp3;
	temp4=block.temp4;
	
  if ( sekunden == 5)
  {
    	        unsigned char sht_checksum, sht_error=0;
		        sht_value humidity;
		        sht_value temperature;
		        float dewpoint;
	            unsigned char do_cycle=0;
	        do
	        {
		        if (do_cycle>1) _delay_ms(200);
		        do_cycle++;
	  
		        sht_error=sht_softreset(SHT_DATA_PIN);
		        if (sht_error)
                {
//			     usart_write("SHT Error (SoftReset) %i\r\n", sht_error);
			     break;
		        }

		        sht_error=sht_measure(&humidity, &sht_checksum, HUMI, SHT_DATA_PIN);
		        if (sht_error)
                {
//			     usart_write("SHT Error (HUMI) %i\r\n", sht_error);
			     break;
		        }
	  
		        sht_error=sht_measure(&temperature, &sht_checksum, TEMP, SHT_DATA_PIN);
		        if (sht_error)
                {
//			    usart_write("SHT Error (TEMP) %i\r\n", sht_error);
			    break;
		        }
   
		        sht_raw_to_physical(&humidity, &temperature);
		
		        if (do_cycle>=5) break;
	        }
	        while ( (humidity.f<100) || (humidity.f>9900) || (temperature.f<-3900) || (temperature.f>10000) );
		
  	
		    dewpoint      = calc_dewpoint(humidity.f,temperature.f);
		
		    temperature.f = temperature.f*100;
		    humidity.f    = humidity.f*100;
		    dewpoint      = dewpoint*100;
		
		    temperatur = temperature.f;
		    feuchtigkeit = humidity.f;
		    taupunkt = dewpoint;
	}		

// 		 anzeige();

  
    switch (sekunden)  // damit der Vorgang nur einmal durchlaufen wird
	{
     case 0:
	 anzeigen = anzeigen * (-1);
		if ( anzeigen == 1)
        {
		 anzeige();
	     anzeigen = 0;
		}
	    break;
		
     case 20:
	 anzeigen = anzeigen * (-1);
		if ( anzeigen == 1)
        {
		 anzeige();
	     anzeigen = 0;
		}
	    break;
		
     case 40:
	 anzeigen = anzeigen * (-1);
		if ( anzeigen == 1)
        {
		 anzeige();
	     anzeigen = 0;
		}
	    break;
		
	 default:                   // Anweisungen wenn keine der oben definierten Bedingungen erfllt ist
      anzeigen = (-1);
     break;
		
		
    }
	
	
  }	
	return 0;
}

