#include "config.h"	       //IMPORTANT: it contains F_CPU definition, so include it before delay.h
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "rfm12.h"
#include <util/delay.h>

//je nach Schaltung anzupassen:
#ifndef RFXX_SEL
#define PORT_SEL PORTB
#define PIN_SEL PINB
#define DDR_SEL DDRB
#define PORT_SDI PORTB
#define PIN_SDI PINB
#define DDR_SDI DDRB
#define PORT_SCK PORTB
#define PIN_SCK PINB
#define DDR_SCK DDRB
#define PORT_SDO PORTB
#define PIN_SDO PINB
#define DDR_SDO DDRB
#define PORT_IRQ PORTD
#define PIN_IRQ PIND
#define DDR_IRQ DDRD
#define PORT_DATA PORTD
#define PIN_DATA PIND
#define DDR_DATA DDRD
#define RFXX_SCK 5// |
#define RFXX_SDO 4// |RF_PORT
#define RFXX_SDI 3// |
#define RFXX_SEL 2// |
#define RF12_IRQ 2
#define RF12_DATA 7
#endif

//Rest sollte ohne weitere Anpassungen laufen:
#define SEL_OUTPUT() DDR_SEL |= (1<<RFXX_SEL)
#define HI_SEL()    PORT_SEL |= (1<<RFXX_SEL)
#define LOW_SEL()   PORT_SEL &= ~(1<<RFXX_SEL)
#define SDI_OUTPUT() DDR_SDI |= (1<<RFXX_SDI)
#define HI_SDI()    PORT_SDI |= (1<<RFXX_SDI)
#define LOW_SDI()   PORT_SDI &= ~(1<<RFXX_SDI)
#define SDO_INPUT()  DDR_SDO &= ~(1<<RFXX_SDO)
#define SCK_OUTPUT() DDR_SCK |= (1<<RFXX_SCK)
#define HI_SCK()    PORT_SCK |= (1<<RFXX_SCK)
#define LOW_SCK()   PORT_SCK &= ~(1<<RFXX_SCK)
#define IRQ_INPUT()  DDR_IRQ &= ~(1<<RF12_IRQ)
//#define DATA_OUTPUT() DDR_DATA |= (1<<RF12_DATA)
//#define HI_DATA()    PORT_DATA |= (1<<RF12_DATA)

void RFXX_PORT_INIT(void)
{
 HI_SEL();
 HI_SDI();
 LOW_SCK();
// HI_DATA(); //SET nFFS pin HI when using FIFO
 SEL_OUTPUT();
 SDI_OUTPUT();
 SDO_INPUT();
 SCK_OUTPUT();
 IRQ_INPUT();
// DATA_OUTPUT();
	unsigned char i;	
	for (i=0; i<10; i++)
		_delay_ms(10);			// wait until POR done

}

unsigned int RFXX_WRT_CMD(unsigned int aCmd)
{
 unsigned char i;
 unsigned int temp=0;
 LOW_SCK();
 LOW_SEL();
 for(i=0;i<16;i++)
   {
    if(aCmd & 0x8000)  HI_SDI();
    else               LOW_SDI();
    HI_SCK();
    temp <<= 1;
    if(PIN_SDO & (1<<RFXX_SDO))  temp |= 0x0001;
    LOW_SCK();
    aCmd <<= 1;
   }
 HI_SEL();
 return(temp);
}

void RF12_INIT(void)
{
 //Erklaerungen zu den Kommentaren:
 // z.B. bei 0x82D9 ist 82 der Befehl, D9 (11011001) sind die Bits,
 // ein ! bedeutet das Bit ist nicht gesetzt, ohne ! ist das Bit gesetzt.
 // Einzelheiten zu den Befehlen und Bits stehen in RF12.pdf
 
/*  RFXX_WRT_CMD(0x80E7);//EL,EF,434MHz,12pF (Befehl 80, D7 = 11010111 = EL,EF,!B1,B0,!x3,x2,x1,x0)
                       //B1,B0: 00=315, 01=434, 10=868, 11=915
                       //(je nach Modul gehen nicht alle Einstellungen!)
                       //x3..x0: (8.5+0.5*x)pF z.B. 8.5+0.5*7=12pF
Beispiele:					   
  RFXX_WRT_CMD(0x80A5); // TX aktiv,868band,11.0pF
  RFXX_WRT_CMD(0xA67C); // 868,3 MHz
  RFXX_WRT_CMD(0xC623); // 9600 Baud !!
  RFXX_WRT_CMD(0xC4A7); // Range limit +7fres to -8fres, Max DEviation +35/-40Khz, Offset VDI=h
  RFXX_WRT_CMD(0xC238); // Datafilter auf analog filter
  RFXX_WRT_CMD(0x9820); // volles ROHR und df(fsk) auf 15KHz
  RFXX_WRT_CMD(0x8279); // Enable baseband, PLL, PA, TX, synthesizer, crystal oscillator, disable clock output
  RFXX_WRT_CMD(0xE000); // Wake-up Timer off
  RFXX_WRT_CMD(0xC800); // low Duty-Cycle off
  RFXX_WRT_CMD(0xC000); // 1.66MHz,2.2V
  RFXX_WRT_CMD(0x0000); // Status auslesen
*/

  RFXX_WRT_CMD(0xC0E0);			// AVR CLK: 10MHz
  #ifdef FREQ433
  RFXX_WRT_CMD(0x80D7);			// Enable FIFO
  #elif defined FREQ868	
  RFXX_WRT_CMD(0x80E7);			//868MHz, EL, EF, 11.5pF
  #endif
  RFXX_WRT_CMD(0xC2AB);			// Data Filter: internal
  RFXX_WRT_CMD(0xCA81);         // FIFO8,SYNC,!ff,DR
  RFXX_WRT_CMD(0xE000);			// disable wakeuptimer
  RFXX_WRT_CMD(0xC800);			// disable low duty cycle
  RFXX_WRT_CMD(0xC4F7);			// AFC settings: autotuning: -10kHz...+7,5kHz
  RFXX_WRT_CMD(0x0000);			// receive status
}

void rfm12_setbandwidth(unsigned char bandwidth, unsigned char gain, unsigned char drssi)
{
	RFXX_WRT_CMD(0x9400|((bandwidth&7)<<5)|((gain&3)<<3)|(drssi&7));
}

void rfm12_setfreq(unsigned short freq)
{	if (freq<96)				// 430,2400MHz
		freq=96;
	else if (freq>3903)			// 439,7575MHz
		freq=3903;
	RFXX_WRT_CMD(0xA000|freq);
}

void rfm12_setbaud(unsigned short baud)
{
	if (baud<663)
		return;
	if (baud<5400)					// Baudrate= 344827,58621/(R+1)/(1+CS*7)
		RFXX_WRT_CMD(0xC680|((43104/baud)-1));
	else
		RFXX_WRT_CMD(0xC600|((344828UL/baud)-1));
}

void rfm12_setpower(unsigned char power, unsigned char mod)
{	
	RFXX_WRT_CMD(0x9800|(power&7)|((mod&15)<<4));
}


void rfm12_init()
{
 RFXX_PORT_INIT(); //Initialize command port
 RF12_INIT();      //Initialize RF12 chip
}

unsigned char RF12_RECV(void)
{
 unsigned int FIFO_data;
// while(PIN_IRQ & (1<<RF12_IRQ))   ;//warte bis IRQ low ist (das war das Original)

 unsigned long timeout = 100000;
 while( (PIN_IRQ & (1<<RF12_IRQ)) && (--timeout) )    ;//warte bis IRQ low ist (das geht)
  
 RFXX_WRT_CMD(0x0000);
 FIFO_data=RFXX_WRT_CMD(0xB000);
 return(FIFO_data&0xFF);
}

void RF12_SEND(unsigned char aByte)
{
 while(PIN_IRQ & (1<<RF12_IRQ))   ;//wait for previously TX over
 RFXX_WRT_CMD(0xB800+aByte);
}

/** Datenpaket-Definition:
1.Byte = Paketlaenge des Datenblocks inklusive dieses Byte
2.Byte = erstes Nutzdaten-Byte
...
n.Byte = letztes Nutzdaten-Byte
weiteres Byte = Pruefsumme = Summe vom 2. bis und mit n. Byte
Minimale und Maximale Paketlaengen sind in sensoren.h definiert.
**/

uchar rfm12_empfang(uchar *data) //Datenpaket empfangen
// data muss fuer MAXPACK Platz haben
// Rueckgabewert: Fehlercode oder 0 wenn alles geklappt hat.
{
 uchar ChkSum=0,c=0,n,err=0;
// RFXX_WRT_CMD(0x0000); //read status register (wirklich noetig?)
 RFXX_WRT_CMD(0x82C8); // RX on
 RFXX_WRT_CMD(0xCA81); // set FIFO mode
 RFXX_WRT_CMD(0xCA83); // Enable FIFO
 *data++ = n = RF12_RECV(); //Paketlaenge
 if(n>=MINPACK && n<=MAXPACK)
  {while(--n!=0)
     {
      ChkSum += *data++ = RF12_RECV(); //Receive payload data
     }
   c=RF12_RECV();
   if(c!=ChkSum) err=ERR_CHKSUM;
  }
 else
  err=ERR_PACKLEN;
 RFXX_WRT_CMD(0x8208); // RX off
 return err;
}

void rfm12_senden(uchar *data) //Datenpaket senden, 1.Byte ist Paketlaenge
{
 uchar ChkSum,c,n;
 RFXX_WRT_CMD(0x8208);			// turn on crystal
 RFXX_WRT_CMD(0x0000);			// receive status
 RFXX_WRT_CMD(0x0000);			// receive status		
 RFXX_WRT_CMD(0x8238);			// TX on
 ChkSum=0;
 RF12_SEND(0xAA);//PREAMBLE
 RF12_SEND(0xAA);//PREAMBLE
 RF12_SEND(0xAA);//PREAMBLE
 RF12_SEND(0x2D);//SYNC HI BYTE
 RF12_SEND(0xD4);//SYNC LOW BYTE
 RF12_SEND(n = *data++); //Paketlaenge
 while(--n!=0)
   {
    RF12_SEND(c = *data++);//DATA BYTE senden
    ChkSum += c;
   }
 RF12_SEND(ChkSum); //send chk sum
 RF12_SEND(0xAA);//DUMMY BYTE
 RF12_SEND(0xAA);//DUMMY BYTE (wirklich noetig?)
 RF12_SEND(0xAA);//DUMMY BYTE (wirklich noetig?)
 _delay_ms(10);
 RFXX_WRT_CMD(0x8208);			// TX off
 RFXX_WRT_CMD(0x8201);			// enter sleep
}

/**
Anwendung im Hauptprogramm:

#include "rfm12_modul.h"
//#include "rfm12_modul.c"

int main(void)
{
 datenblock block;
 antwortblock antwort;
 ...
 Delay_ms(200); //200 Millisekunden warten
 rfm12_init();
 ...
 while(1) //Hauptschlaufe
   {
    ...
    err=rfm12_empfang(block.data);
    if(err==0)
     {id=block.id; //id des Senders lesen
      ...
      antwort.n=4; //Paketlaenge setzen
      antwort.id=myid;
      antwort.com=COM_OK;
      rfm12_senden(antwort.data);
     }
    else
      fehler_beim_empfang();
    ...
   }
}
**/
