/** \file main.c
	\author Tomasz Ostrowski
	\date August 2007
	
	Main file of SHT11 sensor library test project.
	Compiler: avr-gcc (WinAVR 20060421)
	Based on Sensirion sample code app note, rev. 2.05
	MCU: ATmega8, 4MHz crystal, Vcc = 5V (important for SHT11 calculations)
	sensor: SHT11, DATA with external 10k pullup connected to PD4, SCK connected to PD3
	MCU <-> PC: UART
	HFUSE/LFUSE: 0xC9/0x9F
*/

#include "config.h"	       //IMPORTANT: it contains F_CPU definition, so include it before delay.h
#include <avr/sleep.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>


#include "sht/libsht.h"

#include "rfm12/rfm12.h"

static unsigned int sekunden = 0;
float vcc;

// Starten des Timers, nachdem er in der Hauptschleife konfiguriert wurde
uint8_t timer_start(void)
{
 TIMSK1 = (1<<OCIE1A);   //Interrupt bei Compare Match
 sei();            //Interrputs aktivieren
 return 0;
}
  
ISR(TIMER1_COMPA_vect)
{
 sekunden++;
}

//Bei einem unerwarteten Interrupt nichts machen:
void __attribute__ ((naked)) __vector_default(void) {__asm__ ("reti");}

// Batteriespannung messen
void setup_adc(void)
{
 ADMUX = (0<<REFS1) | (1<<REFS0) | (1<<MUX3) | (1<<MUX2) | (1<<MUX1) | (0<<MUX0);      // interne Referenzspannung nutzen
 ADCSRA = (1<<ADEN)|(1<<ADATE)|(1<<ADIE)|(1<<ADSC)|5;
}

ISR(ADC_vect) //ADC End of Conversion interrupt 
{
unsigned char adc_data;
adc_data = ADC>>2; //read 8 bit value
vcc = 1.23 * 25500 / adc_data;
vcc = vcc - 28; // Korrektur
}

void read_adc (void)
{
 sei(); //Activate interrupts
 setup_adc();
 cli();
}

int main(void)
{

// uchar altsekunden;
 datenblock block;

 rfm12_init();
 _delay_ms(500);							// Initialisierung muss manchmal mehrmals gemacht werden,
 rfm12_init();								// da Modul lange braucht
 _delay_ms(500);							// Initialisierung muss manchmal mehrmals gemacht werden,
 rfm12_init();

//rfm12_setfreq(RF12FREQ(433.92));	        // Sende/Empfangsfrequenz auf 433,92MHz einstellen
 rfm12_setfreq(RF12FREQ(868.90));	
 rfm12_setbandwidth(4, 1, 4);			    // 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -79dBm 
 rfm12_setbaud(4800);						// 19200 baud
 rfm12_setpower(0, 6);						// 1mW Ausgangangsleistung, 120kHz Frequenzshift

 
//16Bit-Timer fr Zeitzhlung
  TCCR1B |= (1 << WGM12);                //CTC Modus
  TCCR1B |= (1 << CS12) | (0 << CS11) | (0 << CS10);   //Prescaler 256
  OCR1A = 31249;                      //Compare Wert setzen
  timer_start();
  
//Bei einem unerwarteten Interrupt nichts machen:
void __attribute__ ((naked)) __vector_default(void) {__asm__ ("reti");}
  
  
//	sei(); //interrupts freigeben

    while (1) 
    {
	
     read_adc();
   
      if(sekunden >= 60) sekunden = 0;
	  
	  
		        unsigned char sht_checksum, sht_error=0;
		        sht_value humidity;
		        sht_value temperature;
		        float dewpoint;
	            unsigned char do_cycle=0;
	        do
	        {
		        if (do_cycle>1) _delay_ms(200);
		        do_cycle++;
	  
		        sht_error=sht_softreset(SHT_DATA_PIN);
/*		        if (sht_error)
                {
//			     usart_write("SHT Error (SoftReset) %i\r\n", sht_error);
			     break;
		        }
*/
		        sht_error=sht_measure(&humidity, &sht_checksum, HUMI, SHT_DATA_PIN);
/*		        if (sht_error)
                {
//			     usart_write("SHT Error (HUMI) %i\r\n", sht_error);
			     break;
		        }
*/	  
		        sht_error=sht_measure(&temperature, &sht_checksum, TEMP, SHT_DATA_PIN);
/*		        if (sht_error)
                {
//			    usart_write("SHT Error (TEMP) %i\r\n", sht_error);
			    break;
		        }
*/   
		        sht_raw_to_physical(&humidity, &temperature);
		
		        if (do_cycle>=5) break;
	        }
	        while ( (humidity.f<100) || (humidity.f>9900) || (temperature.f<-3900) || (temperature.f>10000) );
		
		
		    dewpoint      = calc_dewpoint(humidity.f,temperature.f);
		
		    temperature.f = temperature.f*100;
		    humidity.f    = humidity.f*100;
		    dewpoint      = dewpoint*100;
		
            float temperatur, feuchtigkeit,taupunkt;

		    temperatur = temperature.f;
		    feuchtigkeit = humidity.f;
		    taupunkt = dewpoint;
 // 		_delay_ms(200);
 
 
        if ( (sekunden <= 3) )
        {
	  
         block.n=16; //Blocklaenge
         block.id=ID1;
	     float temp1 = temperatur;
	     float temp2 = feuchtigkeit;
	     float temp3 = taupunkt;
	     float temp4 = vcc;
	  
	     block.temp1=temp1;
         block.temp2=temp2;
         block.temp3=temp3;
         block.temp4=temp4;
         rfm12_senden(block.data);
    	}
/*	  
    //moegliche Schlafmoden: SLEEP_MODE_IDLE, _PWR_DOWN, _PWR_SAVE, _ADC, _STANDBY, _EXT_STANDBY
    set_sleep_mode(SLEEP_MODE_PWR_SAVE);
    {
	 altsekunden=sekunden;
	 while(altsekunden==sekunden) sleep_mode(); //naechste Minute abwarten
	}
*/
    }
	return 0;
}
